@@ -1,8 +1,10 @@
 /**************************************************************************
- * 
+ *
  * Copyright 2008 Tungsten Graphics, Inc., Cedar Park, Texas.
+ * Copyright 2009-2010 Chia-I Wu <olvaffe@gmail.com>
+ * Copyright 2010-2011 LunarG, Inc.
  * All Rights Reserved.
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the
  * "Software"), to deal in the Software without restriction, including
@@ -10,19 +12,19 @@
  * distribute, sub license, and/or sell copies of the Software, and to
  * permit persons to whom the Software is furnished to do so, subject to
  * the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice (including the
  * next paragraph) shall be included in all copies or substantial portions
  * of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
- * IN NO EVENT SHALL TUNGSTEN GRAPHICS AND/OR ITS SUPPLIERS BE LIABLE FOR
- * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
- * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
- * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- * 
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ *
  **************************************************************************/
 
 
@@ -36,6 +38,8 @@
 #include "eglcurrent.h"
 #include "eglmisc.h"
 #include "egldisplay.h"
+#include "egldriver.h"
+#include "eglstring.h"
 
 
 /**
@@ -45,7 +49,7 @@ static EGLint
 _eglAppendExtension(char **str, const char *ext)
 {
    char *s = *str;
-   EGLint len = strlen(ext);
+   size_t len = strlen(ext);
 
    if (s) {
       memcpy(s, ext, len);
@@ -58,7 +62,7 @@ _eglAppendExtension(char **str, const char *ext)
       len++;
    }
 
-   return len;
+   return (EGLint) len;
 }
 
 
@@ -73,17 +77,21 @@ _eglUpdateExtensionsString(_EGLDisplay *dpy)
    do {                                                                    \
       if (dpy->Extensions.ext) {                                           \
          _eglAppendExtension(&exts, "EGL_" #ext);                          \
-         assert(exts <= dpy->Extensions.String + _EGL_MAX_EXTENSIONS_LEN); \
+         assert(exts <= dpy->ExtensionsString + _EGL_MAX_EXTENSIONS_LEN);  \
       }                                                                    \
    } while (0)
 
-   char *exts = dpy->Extensions.String;
+   char *exts = dpy->ExtensionsString;
 
    if (exts[0])
       return;
 
    _EGL_CHECK_EXTENSION(MESA_screen_surface);
    _EGL_CHECK_EXTENSION(MESA_copy_context);
+   _EGL_CHECK_EXTENSION(MESA_drm_display);
+   _EGL_CHECK_EXTENSION(MESA_drm_image);
+
+   _EGL_CHECK_EXTENSION(WL_bind_wayland_display);
 
    _EGL_CHECK_EXTENSION(KHR_image_base);
    _EGL_CHECK_EXTENSION(KHR_image_pixmap);
@@ -96,6 +104,15 @@ _eglUpdateExtensionsString(_EGLDisplay *dpy)
    _EGL_CHECK_EXTENSION(KHR_gl_texture_3D_image);
    _EGL_CHECK_EXTENSION(KHR_gl_renderbuffer_image);
 
+   _EGL_CHECK_EXTENSION(KHR_reusable_sync);
+   _EGL_CHECK_EXTENSION(KHR_fence_sync);
+
+   _EGL_CHECK_EXTENSION(KHR_surfaceless_gles1);
+   _EGL_CHECK_EXTENSION(KHR_surfaceless_gles2);
+   _EGL_CHECK_EXTENSION(KHR_surfaceless_opengl);
+
+   _EGL_CHECK_EXTENSION(NOK_swap_region);
+   _EGL_CHECK_EXTENSION(NOK_texture_from_pixmap);
 #undef _EGL_CHECK_EXTENSION
 }
 
@@ -103,24 +120,24 @@ _eglUpdateExtensionsString(_EGLDisplay *dpy)
 static void
 _eglUpdateAPIsString(_EGLDisplay *dpy)
 {
-   char *apis = dpy->ClientAPIs;
+   char *apis = dpy->ClientAPIsString;
 
-   if (apis[0] || !dpy->ClientAPIsMask)
+   if (apis[0] || !dpy->ClientAPIs)
       return;
 
-   if (dpy->ClientAPIsMask & EGL_OPENGL_BIT)
+   if (dpy->ClientAPIs & EGL_OPENGL_BIT)
       strcat(apis, "OpenGL ");
 
-   if (dpy->ClientAPIsMask & EGL_OPENGL_ES_BIT)
+   if (dpy->ClientAPIs & EGL_OPENGL_ES_BIT)
       strcat(apis, "OpenGL_ES ");
 
-   if (dpy->ClientAPIsMask & EGL_OPENGL_ES2_BIT)
+   if (dpy->ClientAPIs & EGL_OPENGL_ES2_BIT)
       strcat(apis, "OpenGL_ES2 ");
 
-   if (dpy->ClientAPIsMask & EGL_OPENVG_BIT)
+   if (dpy->ClientAPIs & EGL_OPENVG_BIT)
       strcat(apis, "OpenVG ");
 
-   assert(strlen(apis) < sizeof(dpy->ClientAPIs));
+   assert(strlen(apis) < sizeof(dpy->ClientAPIsString));
 }
 
 
@@ -128,51 +145,23 @@ const char *
 _eglQueryString(_EGLDriver *drv, _EGLDisplay *dpy, EGLint name)
 {
    (void) drv;
-   (void) dpy;
+
    switch (name) {
    case EGL_VENDOR:
       return _EGL_VENDOR_STRING;
    case EGL_VERSION:
-      return dpy->Version;
+      _eglsnprintf(dpy->VersionString, sizeof(dpy->VersionString),
+              "%d.%d (%s)", dpy->VersionMajor, dpy->VersionMinor,
+              dpy->Driver->Name);
+      return dpy->VersionString;
    case EGL_EXTENSIONS:
       _eglUpdateExtensionsString(dpy);
-      return dpy->Extensions.String;
-#ifdef EGL_VERSION_1_2
+      return dpy->ExtensionsString;
    case EGL_CLIENT_APIS:
       _eglUpdateAPIsString(dpy);
-      return dpy->ClientAPIs;
-#endif
+      return dpy->ClientAPIsString;
    default:
       _eglError(EGL_BAD_PARAMETER, "eglQueryString");
       return NULL;
    }
 }
-
-
-EGLBoolean
-_eglWaitClient(_EGLDriver *drv, _EGLDisplay *dpy, _EGLContext *ctx)
-{
-   /* just a placeholder */
-   (void) drv;
-   (void) dpy;
-   (void) ctx;
-   return EGL_TRUE;
-}
-
-
-EGLBoolean
-_eglWaitNative(_EGLDriver *drv, _EGLDisplay *dpy, EGLint engine)
-{
-   /* just a placeholder */
-   (void) drv;
-   (void) dpy;
-   switch (engine) {
-   case EGL_CORE_NATIVE_ENGINE:
-      break;
-   default:
-      _eglError(EGL_BAD_PARAMETER, "eglWaitNative(engine)");
-      return EGL_FALSE;
-   }
-
-   return EGL_TRUE;
-}
