@@ -40,7 +40,6 @@
 #include "i915_reg.h"
 #include "i915_program.h"
 
-#include "intel_tris.h"
 #include "intel_span.h"
 
 /***************************************
@@ -50,7 +49,7 @@
 /* Override intel default.
  */
 static void
-i915InvalidateState(GLcontext * ctx, GLuint new_state)
+i915InvalidateState(struct gl_context * ctx, GLuint new_state)
 {
    _swrast_InvalidateState(ctx, new_state);
    _swsetup_InvalidateState(ctx, new_state);
@@ -70,8 +69,6 @@ i915InvalidateState(GLcontext * ctx, GLuint new_state)
          p->params_uptodate = 0;
    }
 
-   if (new_state & (_NEW_FOG | _NEW_HINT | _NEW_PROGRAM | _NEW_PROGRAM_CONSTANTS))
-      i915_update_fog(ctx);
    if (new_state & (_NEW_STENCIL | _NEW_BUFFERS | _NEW_POLYGON))
       i915_update_stencil(ctx);
    if (new_state & (_NEW_LIGHT))
@@ -95,7 +92,7 @@ extern const struct tnl_pipeline_stage *intel_pipeline[];
 
 GLboolean
 i915CreateContext(int api,
-		  const __GLcontextModes * mesaVis,
+		  const struct gl_config * mesaVis,
                   __DRIcontext * driContextPriv,
                   void *sharedContextPrivate)
 {
@@ -103,14 +100,11 @@ i915CreateContext(int api,
    struct i915_context *i915 =
       (struct i915_context *) CALLOC_STRUCT(i915_context);
    struct intel_context *intel = &i915->intel;
-   GLcontext *ctx = &intel->ctx;
+   struct gl_context *ctx = &intel->ctx;
 
    if (!i915)
       return GL_FALSE;
 
-   if (0)
-      printf("\ntexmem-0-3 branch\n\n");
-
    i915InitVtbl(i915);
 
    i915InitDriverFunctions(&functions);
@@ -197,6 +191,7 @@ i915CreateContext(int api,
       & ctx->ShaderCompilerOptions[MESA_SHADER_FRAGMENT];
    fs_options->EmitNoIfs = GL_TRUE;
    fs_options->EmitNoNoise = GL_TRUE;
+   fs_options->EmitNoPow = GL_TRUE;
    fs_options->EmitNoMainReturn = GL_TRUE;
    fs_options->EmitNoIndirectInput = GL_TRUE;
    fs_options->EmitNoIndirectOutput = GL_TRUE;
@@ -212,5 +207,11 @@ i915CreateContext(int api,
 
    i915InitState(i915);
 
+   /* Always enable pixel fog.  Vertex fog using fog coord will conflict
+    * with fog code appended onto fragment program.
+    */
+   _tnl_allow_vertex_fog(ctx, 0);
+   _tnl_allow_pixel_fog(ctx, 1);
+
    return GL_TRUE;
 }
