@@ -256,7 +256,7 @@ struct sis_renderbuffer {
 struct sis_context
 {
   /* This must be first in this structure */
-  GLcontext *glCtx;
+  struct gl_context *glCtx;
 
   /* Vertex state */
   GLuint vertex_size;
@@ -439,12 +439,12 @@ enum _sis_verbose {
 };
 
 extern GLboolean sisCreateContext( gl_api api,
-				   const __GLcontextModes *glVisual,
+				   const struct gl_config *glVisual,
 				   __DRIcontext *driContextPriv,
                                    void *sharedContextPrivate );
 extern void sisDestroyContext( __DRIcontext * );
 
-void sisReAllocateBuffers(GLcontext *ctx, GLframebuffer *drawbuffer,
+void sisReAllocateBuffers(struct gl_context *ctx, struct gl_framebuffer *drawbuffer,
                           GLuint width, GLuint height);
 
 extern GLboolean sisMakeCurrent( __DRIcontext *driContextPriv,
